productname = 'PyGopherd'
versionstr = "2.0.18"

versionlist = versionstr.split(".")
major = versionlist[0]
minor = versionlist[1]
patch = versionlist[2]
copyright = "Copyright (C) 2002-2008 John Goerzen.  All rights reserved."
author = "John Goerzen"
author_email = "jgoerzen@complete.org"
description = "Multiprotocol Internet Gopher Information Server"
homepage = "http://www.quux.org/devel/gopher/pygopherd"
homegopher = "gopher://quux.org/1/devel/gopher/pygopherd"
license = """Copyright (C) 2002-2008 John Goerzen <jgoerzen@complete.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA"""
